CipherState

Struct CipherState 

pub struct CipherState<C: Cipher> { /* private fields */ }
Cipherstate for encrypting and decrypting messages

Contains the encryption key and nonce and provides methods for encrypting and decrypting data. Will automatically increment the nonce and return an error if that overflows.

Implementations

Source

impl<C: Cipher> CipherState<C>

pub fn new(k: &[u8], n: u64) -> Self

Initialize with given key and nonce

Panics

Panics if key data has incorrect length

pub fn encrypt_with_ad( &mut self, ad: &[u8], plaintext: &[u8], out: &mut [u8], ) -> CipherResult<()>

AEAD encryption

pub fn encrypt_with_ad_in_place( &mut self, ad: &[u8], in_out: &mut [u8], plaintext_len: usize, ) -> CipherResult<usize>

AEAD encryption in place

pub fn decrypt_with_ad( &mut self, ad: &[u8], ciphertext: &[u8], out: &mut [u8], ) -> CipherResult<()>

AEAD decryption

pub fn decrypt_with_ad_in_place( &mut self, ad: &[u8], in_out: &mut [u8], ciphertext_len: usize, ) -> CipherResult<usize>

AEAD decryption in place

pub fn get_nonce(&self) -> u64

Get current nonce value

pub fn set_nonce(&mut self, nonce: u64)

Set nonce value

Warning

Do not reuse nonces. Doing so WILL LEAD to a catastrophic crypto failure.

pub fn take(self) -> (C::Key, u64)

Take ownership of key and nonce of this state

Warning

Use with care

pub fn rekey(&mut self) -> CipherResult<()>

Rekey

Rekeys as per Noise spec parts 4.2 and 11.3