pub struct CipherState<C: Cipher> { /* private fields */ }Expand description
Cipherstate for encrypting and decrypting messages
Contains the encryption key and nonce and provides methods for encrypting and decrypting data. Will automatically increment the nonce and return an error if that overflows.
Implementations
Sourceimpl<C: Cipher> CipherState<C>
impl<C: Cipher> CipherState<C>
Sourcepub fn encrypt_with_ad(
&mut self,
ad: &[u8],
plaintext: &[u8],
out: &mut [u8],
) -> CipherResult<()>
pub fn encrypt_with_ad( &mut self, ad: &[u8], plaintext: &[u8], out: &mut [u8], ) -> CipherResult<()>
AEAD encryption
Sourcepub fn encrypt_with_ad_in_place(
&mut self,
ad: &[u8],
in_out: &mut [u8],
plaintext_len: usize,
) -> CipherResult<usize>
pub fn encrypt_with_ad_in_place( &mut self, ad: &[u8], in_out: &mut [u8], plaintext_len: usize, ) -> CipherResult<usize>
AEAD encryption in place
Sourcepub fn decrypt_with_ad(
&mut self,
ad: &[u8],
ciphertext: &[u8],
out: &mut [u8],
) -> CipherResult<()>
pub fn decrypt_with_ad( &mut self, ad: &[u8], ciphertext: &[u8], out: &mut [u8], ) -> CipherResult<()>
AEAD decryption
Sourcepub fn decrypt_with_ad_in_place(
&mut self,
ad: &[u8],
in_out: &mut [u8],
ciphertext_len: usize,
) -> CipherResult<usize>
pub fn decrypt_with_ad_in_place( &mut self, ad: &[u8], in_out: &mut [u8], ciphertext_len: usize, ) -> CipherResult<usize>
AEAD decryption in place
Sourcepub fn rekey(&mut self) -> CipherResult<()>
pub fn rekey(&mut self) -> CipherResult<()>
Rekey
Rekeys as per Noise spec parts 4.2 and 11.3